package edu.georgetown.gucs.dictionary;

import java.io.File;
import java.util.Iterator;
import java.util.List;
import java.util.TreeMap;

import edu.georgetown.gucs.tokenizers.TokenizerList;

/**
 * Thread for processing a file to add to a {@linkplain Dictionary}
 * 
 * @author Clay Shields
 * 
 **/
public class DictionaryWorker implements Runnable {

  private File source;
  private List<String> tokenizerNames;
  private Dictionary dictionary;
  private TokenizerList tokenizers;

  /**
   * Constructor that specifies a file to tokenize and add to an existing dictionary
   * 
   * @param filename
   *          the file to tokenize and add to this dictionary
   * @param tokenizers
   *          the list containing the names of the tokenizers to use on this file
   * @param dict
   *          the dictionary to add the file to
   */
  public DictionaryWorker(File filename, List<String> tokenizers, Dictionary dict) {
    source = filename;
    tokenizerNames = tokenizers;
    dictionary = dict;
  }

  /**
   * Adds a file to an existing dictionary using the given tokenizers
   */
  public void run() {

    tokenizers = new TokenizerList(tokenizerNames);

    // iterate over all terms and insert them into a temporary tree map
    TreeMap<String, DictionaryEntry> fileInput = new TreeMap<String, DictionaryEntry>();
    Iterator<String> tokens = tokenizers.tokenizeFile(source);

    while (tokens.hasNext()) {
      // get a token
      String token = tokens.next().trim();

      if (token.length() > 0) {
        // see if it is already in the temporary tree map
        if (fileInput.containsKey(token)) {
          // if so, increment the frequency count
          fileInput.get(token).incrementFrequencyCount();
        } else {
          // otherwise, create a new DictionaryEntry object for the token and
          // insert it
          DictionaryEntry tobeinserted = new DictionaryEntry(token);
          fileInput.put(token, tobeinserted);
        }
      }
    }
    dictionary.mergeDictionary(fileInput);
  }
}
