#include <jni.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include <stdlib.h>
#include <stdio.h>

#define SCC_FILENAME_MAX  1024
#include "sccta.h"
#include "sccio.h"

#include "OutsideInFileTokenizer.h"
#include "outsideInExtract.h"

jobjectArray metaDataArray;
int* bytes;

/**
 * Extracts text from a file and splits it into tokens based on whitespace or by line
 *
 * @param filename	the jstring filename of the document to split into tokens
 * @param lines		the jint that specifies whether to split tokens based on
 * 					whitespace (lines=0) or by line (lines=1)
 *
 * @return the jobjectArray containing the tokens from the file
 */
JNIEXPORT jobjectArray JNICALL Java_edu_georgetown_gucs_tokenizers_OutsideInFileTokenizer_extractText
(JNIEnv *env, jobject jobj, jstring filename, jint lines) {

	const char *mfile = (*env)->GetStringUTFChars(env, filename, 0);
	char** tokens = runTextExtraction(mfile, lines);
	if (tokens == NULL) {
		printf("[JNI] tokens = NULL for filename: %s\n", mfile);
		return NULL;
	}

	// positions int array
	bytes = getByteResult();

	jclass string_class = (*env)->FindClass(env, "java/lang/String");
	jint metaDataArray_size = tokens[1];
	jint tokenArray_size = tokens[0];
	jobjectArray tokenArray = (*env)->NewObjectArray(env, tokenArray_size, string_class, NULL);
	metaDataArray = (*env)->NewObjectArray(env, metaDataArray_size, string_class, NULL);

	jint size;
	for (size=2; size <= tokenArray_size+metaDataArray_size && tokens[size] != NULL; size++) {
		if (size <= metaDataArray_size) {
			(*env)->SetObjectArrayElement(env, metaDataArray, size-2, (*env)->NewStringUTF(env, tokens[size]));
		} else {
			(*env)->SetObjectArrayElement(env, tokenArray, size-metaDataArray_size-1, (*env)->NewStringUTF(env, tokens[size]));
		}
	}
	(*env)->ReleaseStringUTFChars(env, filename, mfile);
	return tokenArray;
}

/**
 * Prints to the screen a portion of the given document
 *
 * @param filename	the jstring filename of the document to split into tokens
 * @param start		the number of characters to skip in file before printing, inclusive
 * @param end		the number of characters to print from the file, exclusive
 */
JNIEXPORT void JNICALL Java_edu_georgetown_gucs_tokenizers_OutsideInFileTokenizer_getIndex
(JNIEnv *env, jobject jobj, jstring filename, jint start, jint end) {
	const char *mfile = (*env)->GetStringUTFChars(env, filename, 0);
	printIndex(mfile, start, end);
	(*env)->ReleaseStringUTFChars(env, filename, mfile);
}


/**
 * Provides an integer array of the position of each token. The returned array
 * is initialized and populated in the extractText method and must therefore
 * only be called after a successful call to that method.
 *
 * @return the jobjectArray of integer positions of each token
 */
JNIEXPORT jobjectArray JNICALL Java_edu_georgetown_gucs_tokenizers_OutsideInFileTokenizer_getPositions
(JNIEnv *env, jobject jobj) {
	jint bytes_size = 0;
	while(bytes[bytes_size] != NULL) {
		bytes_size++;
	}
	jintArray positionArray = (*env)->NewIntArray(env, bytes_size);
	(*env)->SetIntArrayRegion(env, positionArray, 0, bytes_size, (jint*) bytes);
	return positionArray;
}

/**
 * Returns meta data tokens from the text extracted. The returned string array
 * is initialized and populated in the extractText method and must therefore
 * only be called after a successful call to that method.
 *
 * @return the string array containing the meta data tokens from the file
 */
JNIEXPORT jobjectArray JNICALL Java_edu_georgetown_gucs_tokenizers_OutsideInFileTokenizer_getMetaData
(JNIEnv *env, jobject jobj) {
	return metaDataArray;
}
