package edu.georgetown.gucs.tokenizers;

import java.util.Iterator;
import java.util.Locale;

import edu.georgetown.gucs.utility.Pair;

/**
 * Uses English stemming to change tokens into their root form
 * 
 * @author Clay Shields
 * @author Lindsay Neubauer
 * 
 */
public class PorterTokenizer extends Tokenizer {

  /**
   * Changes English language tokens into their root form
   */
  public void tokenize(Iterator<String> iterator) {
    tokenVector.clear();
    Stemmer porterStemmer = new Stemmer();
    while (iterator.hasNext()) {
      String lc_token = iterator.next().trim().toLowerCase(Locale.ENGLISH);
      for (int i = 0; i < lc_token.length(); i++) {
        porterStemmer.add(lc_token.charAt(i));
      }
      porterStemmer.stem();
      String porterized = porterStemmer.toString();
      tokenVector.add(porterized);
    }
  }

  /**
   * Changes English language tokens into their root form
   */
  public void tokenize(Iterator<String> tokensIterator, Iterator<Pair<Integer, Integer>> positionsIterator) {
    tokenVector.clear();
    positions.clear();
    Stemmer porterStemmer = new Stemmer();
    while (tokensIterator.hasNext() && positionsIterator.hasNext()) {
      String lc_token = tokensIterator.next().trim().toLowerCase(Locale.ENGLISH);
      Pair<Integer, Integer> pos = positionsIterator.next();
      for (int i = 0; i < lc_token.length(); i++) {
        porterStemmer.add(lc_token.charAt(i));
      }
      porterStemmer.stem();
      String porterized = porterStemmer.toString();
      tokenVector.add(porterized);
      positions.add(pos);
    }
  }
}
