package edu.georgetown.gucs.tokenizers;

import java.util.Iterator;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import edu.georgetown.gucs.utility.Pair;

/**
 * Eliminates tokens that are numbers
 * 
 * @author Clay Shields
 * @author Lindsay Neubauer
 * 
 */
public class RemoveNumericTokensTokenizer extends Tokenizer {

  private Pattern p;
  private Matcher m;

  /**
   * Constructor that sets the Pattern to remove numeric tokens
   */
  public RemoveNumericTokensTokenizer() {
    p = Pattern.compile("^\\d+$");
  }

  /**
   * Eliminates tokens that are numbers
   */
  public void tokenize(Iterator<String> iterator) {
    tokenVector.clear();
    while (iterator.hasNext()) {
      String token = iterator.next().trim();
      m = p.matcher(token);
      if (!m.matches()) {
        tokenVector.add(token);
      }
    }
  }

  /**
   * Eliminates tokens that are numbers
   */
  public void tokenize(Iterator<String> tokensIterator, Iterator<Pair<Integer, Integer>> positionsIterator) {
    tokenVector.clear();
    positions.clear();
    while (tokensIterator.hasNext() && positionsIterator.hasNext()) {
      String token = tokensIterator.next().trim();
      Pair<Integer, Integer> pos = positionsIterator.next();
      m = p.matcher(token);
      if (!m.matches()) {
        tokenVector.add(token);
        positions.add(pos);
      }
    }
  }
}
