package edu.georgetown.gucs.tokenizers;

import java.util.Iterator;

import edu.georgetown.gucs.utility.Pair;

/**
 * Separates tokens based on punctuation and removes punctuation from tokens
 * 
 * @author Clay Shields
 * @author Lindsay Neubauer
 * 
 */
public class StripPunctuationTokenizer extends Tokenizer {

  // This is the posix format for punctuation characters
  private String delimiters = "\\p{Punct}";

  /**
   * Separates tokens based on punctuation and removes punctuation from tokens
   */
  public void tokenize(Iterator<String> iterator) {
    tokenVector.clear();
    while (iterator.hasNext()) {
      for (String token : iterator.next().split(delimiters, 0)) {
        if (token.length() > 0) {
          tokenVector.add(token.trim());
        }
      }
    }
  }

  /**
   * Separates tokens based on punctuation and removes punctuation from tokens
   */
  public void tokenize(Iterator<String> tokensIterator, Iterator<Pair<Integer, Integer>> positionsIterator) {
    tokenVector.clear();
    positions.clear();
    while (tokensIterator.hasNext() && positionsIterator.hasNext()) {
      Pair<Integer, Integer> pos = positionsIterator.next();
      for (String token : tokensIterator.next().split(delimiters, 0)) {
        if (token.length() > 0) {
          tokenVector.add(token.trim());
          positions.add(pos);
        }
      }
    }
  }
}
